#include <imsls.h>
#include <imsl.h>
#include <stdio.h>
#include <math.h>
static float fcn(int, float[], int, float[]);
static void jacobian(int, float[], int, float[], float[]);
int main()
{
   int df;
   int n_independent=1;
   int n_parameters = 2;
   int n_obs        = 15;
   float *theta_hat, *r, *y_hat, *var_covar;
   float grad_eps = 1.0e-9;
   float theta_guess[2] = {60.0, -0.03};
   float a, dfe, normalValue;
   float y[15] = 
   {
      54.0, 50.0, 45.0, 37.0, 35.0,
      25.0, 20.0, 16.0, 18.0, 13.0,
      8.0, 11.0, 8.0, 4.0, 6.0 
   };
   float x[15] = 
   {
      2.0, 5.0, 7.0, 10.0, 14.0,
      19.0, 26.0, 31.0, 34.0, 38.0,
      45.0, 52.0, 53.0, 60.0, 65.0 
   };
   char *fmt="%15.3f";
   char *dashes=
      "-------------------------------------------------------------";

   /* Nonlinear regression */
   theta_hat = imsls_f_nonlinear_regression(fcn, n_parameters,
      n_obs, n_independent, x, y,
      IMSLS_THETA_GUESS, theta_guess,
      IMSLS_GRADIENT_EPS, grad_eps,
      IMSLS_DF, &df,
      IMSLS_R, &r,
      IMSLS_PREDICTED, &y_hat,
      IMSLS_VARIANCE_COVARIANCE_MATRIX, &var_covar,
      IMSLS_JACOBIAN, jacobian, 0);
   /* Print results */
   /* Calculate and Print Coefficients & their 95% Confidence Limits */
   printf(" \n                   ESTIMATED COEFFICIENTS \n");
   printf("%s\n", dashes);
   printf(" Coefficient | Lower 95%% Limit |");
   printf(" Estimate | Upper 95%% Limit \n");
   dfe = (float) df;
   normalValue = imsls_f_t_inverse_cdf(0.975, dfe);
   a = normalValue * sqrt(var_covar[0]);
   printf("     Theta_1 | %10.3f      | %7.3f  | %12.3f\n", 
         theta_hat[0] - a, theta_hat[0], theta_hat[0] + a);
   a = normalValue * sqrt(var_covar[3]);
   printf("     Theta_2 | %10.3f      | %7.3f  | %12.3f\n", 
         theta_hat[1] - a, theta_hat[1], theta_hat[1] + a);
   printf("%s\n", dashes);
   imsls_f_write_matrix("Var/Covar matrix", n_parameters, n_parameters,
      var_covar, IMSLS_WRITE_FORMAT, fmt, 0);
   imsls_f_write_matrix("Predicted values", 1, n_obs,   y_hat, 
      IMSLS_WRITE_FORMAT, "%7.2f", 0);
      return 0;
}

static float fcn(int n_independent, float x[], 
                 int n_parameters, float theta[])
{
   return (theta[0]*exp(x[0]*theta[1]));
} /* End of fcn */
static void jacobian(int n_independent, float x[], 
                     int n_parameters,  float theta[], 
                     float fjac[])
{
   fjac[0] = exp(theta[1]*x[0]);
   fjac[1] = theta[0]*x[0]*exp(theta[1]*x[0]);
} /* End of jacobian */
